@@ -61,8 +61,6 @@ def laplace(field: GridType,
     Returns:
         laplacian field as `CenteredGrid`
     """
-    if implicit:
-        warnings.warn("Implicit operators currently do not support sparse matrix generation and may be slow.", RuntimeWarning, stacklevel=2)
     if isinstance(weights, Field):
         weights = weights.at(field).values
     axes_names = field.shape.only(axes).names
@@ -134,8 +132,6 @@ def spatial_gradient(field: CenteredGrid,
     Returns:
         spatial_gradient field of type `type`.
     """
-    if implicit:
-        warnings.warn("Implicit operators currently do not support sparse matrix generation and may be slow.", RuntimeWarning, stacklevel=2)
     if gradient_extrapolation is None:
         gradient_extrapolation = field.extrapolation.spatial_gradient()
     extrap_map = {}
@@ -210,8 +206,7 @@ def _ex_map_f(ext_dict: dict):
     return f
 
 
-# @jit_compile_linear(auxiliary_args="values_rhs, needed_shifts_rhs, stack_dim, staggered_output")
-@jit_compile(auxiliary_args="values_rhs, needed_shifts_rhs, stack_dim, staggered_output")  # ToDo the matrix generation gives incorrect results in 2.3.0
+@jit_compile_linear(auxiliary_args="values_rhs, needed_shifts_rhs, stack_dim, staggered_output")
 def _lhs_for_implicit_scheme(x, values_rhs, needed_shifts_rhs, stack_dim, staggered_output=False):
     result = []
     for dim, component in zip(x.shape.only(math.spatial).names, unstack(x, stack_dim.name)):
