@@ -95,11 +95,19 @@ def unstack(value, dim: DimFilter):
             value_packed = value.__pack_dims__(dims.names, packed_dim, pos=None)
             if value_packed is not NotImplemented:
                 return unstack(value_packed, packed_dim)
-        first_unstacked = unstack(value, dims[0])
-        inner_unstacked = [unstack(v, dims.without(dims[0])) for v in first_unstacked]
+        unstack_dim = _any_uniform_dim(dims)
+        first_unstacked = unstack(value, unstack_dim)
+        inner_unstacked = [unstack(v, dims.without(unstack_dim)) for v in first_unstacked]
         return sum(inner_unstacked, ())
 
 
+def _any_uniform_dim(dims: Shape):
+    for dim in dims:
+        if dim.is_uniform:
+            return dim
+    raise ValueError(f"Uniform dimension required but found only non-uniform dimensions {dims}")
+
+
 def stack(values: tuple or list or dict, dim: Shape, expand_values=False, **kwargs):
     """
     Stacks `values` along the new dimension `dim`.
@@ -143,17 +151,17 @@ def stack(values: tuple or list or dict, dim: Shape, expand_values=False, **kwar
             assert set(non_batch(v).names) == set(non_batch(values_[0]).names), f"Stacked values must have the same non-batch dimensions but got {non_batch(values_[0])} and {non_batch(v)}"
     # --- Add missing dimensions ---
     if expand_values:
-        all_dims = merge_shapes(*values_)
+        all_dims = merge_shapes(*values_, allow_varying_sizes=True)
         if isinstance(values, dict):
-            values = {k: expand(v, all_dims.without(shape(v).non_batch)) for k, v in values.items()}
+            values = {k: expand(v, all_dims.without(shape(v))) for k, v in values.items()}
         else:
-            values = [expand(v, all_dims.without(shape(v).non_batch)) for v in values]
+            values = [expand(v, all_dims.without(shape(v))) for v in values]
     else:
-        all_batch_dims = merge_shapes(*[batch(v) for v in values_])
+        all_batch_dims = merge_shapes(*[batch(v) for v in values_], allow_varying_sizes=True)
         if isinstance(values, dict):
-            values = {k: expand(v, all_batch_dims) for k, v in values.items()}
+            values = {k: expand(v, all_batch_dims.without(shape(v))) for k, v in values.items()}
         else:
-            values = [expand(v, all_batch_dims) for v in values]
+            values = [expand(v, all_batch_dims.without(shape(v))) for v in values]
     if dim.rank == 1:
         assert dim.size == len(values) or dim.size is None, f"stack dim size must match len(values) or be undefined but got {dim} for {len(values)} values"
         if dim.size is None:
@@ -354,8 +362,8 @@ def expand(value, *dims: Shape, **kwargs):
 
 
 def rename_dims(value,
-                dims: str or tuple or list or Shape,
-                names: str or tuple or list or Shape,
+                dims: DimFilter,
+                names: DimFilter,
                 **kwargs):
     """
     Change the name and optionally the type of some dimensions of `value`.
@@ -364,11 +372,12 @@ def rename_dims(value,
 
     Args:
         value: `Shape` or `Tensor` or `Shapable`.
-        dims: Existing dimensions of `value`.
+        dims: Existing dimensions of `value` as comma-separated `str`, `tuple`, `list`, `Shape` or filter function.
         names: Either
 
             * Sequence of names matching `dims` as `tuple`, `list` or `str`. This replaces only the dimension names but leaves the types untouched.
             * `Shape` matching `dims` to replace names and types.
+            * Dimension type function to replace only types.
 
         **kwargs: Additional keyword arguments required by specific implementations.
             Adding spatial dimensions to fields requires the `bounds: Box` argument specifying the physical extent of the new dimensions.
@@ -382,9 +391,11 @@ def rename_dims(value,
     elif isinstance(value, (Number, bool)):
         return value
     assert isinstance(value, Shapable) and isinstance(value, Shaped), f"value must be a Shape or Shapable but got {type(value).__name__}"
-    dims = parse_dim_order(dims)
+    dims = shape(value).only(dims).names if callable(dims) else parse_dim_order(dims)
     if isinstance(names, str):
         names = parse_dim_order(names)
+    elif callable(names):
+        names: Shape = names(*dims).with_sizes(shape(value))
     assert len(dims) == len(names), f"names and dims must be of equal length but got #dims={len(dims)} and #names={len(names)}"
     existing_dims = shape(value).only(dims, reorder=True)
     if not existing_dims:
